"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exhaust = void 0;
const types_1 = require("../types");
/** Parses using the passed in parser, until the input is exhausted or until the `until` condition is satisfied.
 * @returns A parser returning an array of parsed results.
 */
function exhaust(parser, until = null) {
    return (ctx) => {
        const results = [];
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const res = parser(ctx);
            if (types_1.isFailure(res)) {
                if (until === null || types_1.isFailure(until(ctx))) {
                    return types_1.failure(res.ctx, res.expected, ['exhaust', ...res.history]);
                }
                return types_1.success(ctx, results);
            }
            ctx = res.ctx;
            results.push(res.value);
            if (res.ctx.index === res.ctx.text.length)
                return types_1.success(res.ctx, results);
        }
    };
}
exports.exhaust = exhaust;
