"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.oneOrManyRed = exports.oneOrMany = exports.zeroOrMany = exports.many = void 0;
const types_1 = require("../types");
const map_1 = require("./map");
const opt_1 = require("./opt");
const seq_1 = require("./seq");
/** Parses zero or more occurences of the given parser.
 * @returns A parser returning an array of many parses.
 */
function many(parser) {
    return (ctx) => {
        const results = [];
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const res = parser(ctx);
            if (types_1.isFailure(res)) {
                return types_1.success(ctx, results);
            }
            ctx = res.ctx;
            results.push(res.value);
        }
    };
}
exports.many = many;
/** Parses zero or more occurences of the given parser, separated with the separator parser.
 * @returns A parser returning an array of many parses, omitting the separator.
 */
function zeroOrMany(item, separator) {
    return map_1.map(opt_1.opt(oneOrMany(item, separator)), t => t ?? []);
}
exports.zeroOrMany = zeroOrMany;
/** Parses one or more occurences of the given parser, separated with the separator parser.
 * @returns A parser returning an array of many parses, omitting the separator.
 */
function oneOrMany(item, separator) {
    return (ctx) => {
        const res = map_1.map(seq_1.seq(item, many(map_1.map(seq_1.seq(separator, item), ([, t]) => t))), ([t, ts]) => ([t, ...ts]))(ctx);
        if (types_1.isFailure(res)) {
            const newHistory = [...res.history];
            newHistory.splice(0, 2);
            return types_1.failure(res.ctx, res.expected, ['oneOrMany', ...newHistory]);
        }
        return res;
    };
}
exports.oneOrMany = oneOrMany;
/** Parses one or more occurences of the given parser, separated with the separator parser.
 * @returns A parser returning the result of many parses, reduced using the `reducer` function passed in.
 */
function oneOrManyRed(item, separator, reducer) {
    return map_1.map(map_1.map(seq_1.seq(map_1.map(item, (x) => [null, x]), many(seq_1.seq(separator, item))), ([t, ts]) => ([t, ...ts])), ts => {
        let result = ts[0][1];
        for (let i = 1; i < ts.length; i++) {
            result = reducer(result, ts[i][1], ts[i][0]);
        }
        return result;
    });
}
exports.oneOrManyRed = oneOrManyRed;
