"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.expect = exports.ref = void 0;
const types_1 = require("../types");
/** Allows to make a condition on the result of the parsing function.
 * @returns A parser returning the same but also performing a given check on the result.
 */
function ref(parser, check, expected) {
    return (ctx) => {
        const res = parser(ctx);
        if (!types_1.isFailure(res) && !check(res.value)) {
            return types_1.failure(res.ctx, expected, [`ref: ${expected}`]);
        }
        return res;
    };
}
exports.ref = ref;
/** Changes the expected value for when the parser fails.
 * @returns A parser returning the same but with a different expected value.
 */
function expect(parser, expected) {
    return (ctx) => {
        const res = parser(ctx);
        if (types_1.isFailure(res)) {
            return types_1.failure(res.ctx, expected, [expected, ...res.history]);
        }
        return res;
    };
}
exports.expect = expect;
