export declare type Parser<T> = (ctx: Context) => Result<T>;
export declare type Context = Readonly<{
    text: string;
    path: string;
    index: number;
}>;
export declare type Result<T> = Success<T> | Failure;
export declare type Success<T> = Readonly<{
    success: true;
    value: T;
    ctx: Context;
}>;
export declare type Failure = Readonly<{
    success: false;
    expected: string;
    ctx: Context;
    history: string[];
}>;
export declare function isFailure(input: unknown): input is Failure;
export declare function success<T>(ctx: Context, value: T): Success<T>;
export declare function failure(ctx: Context, expected: string, history: string[]): Failure;
export declare function result<T>(value: T): Parser<T>;
export declare function fail<T>(reason: string): Parser<T>;
export declare class ParseError extends Error {
    readonly index: number;
    readonly history: string[];
    readonly line: string;
    readonly column: number;
    readonly row: number;
    constructor(message: string, input: string, index: number, history: string[]);
    private getInputData;
    getPrettyErrorMessage(): string;
}
