"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParseError = exports.fail = exports.result = exports.failure = exports.success = exports.isFailure = void 0;
function isFailure(input) {
    return !input.success;
}
exports.isFailure = isFailure;
function success(ctx, value) {
    return { success: true, value, ctx };
}
exports.success = success;
function failure(ctx, expected, history) {
    return { success: false, expected, ctx, history };
}
exports.failure = failure;
function result(value) {
    return (ctx) => success(ctx, value);
}
exports.result = result;
function fail(reason) {
    return (ctx) => failure(ctx, reason, [reason]);
}
exports.fail = fail;
class ParseError extends Error {
    constructor(message, input, index, history) {
        super(message);
        this.index = index;
        this.history = history;
        [this.line, this.column, this.row] = this.getInputData(input, index);
    }
    getInputData(input, index) {
        const lines = input.split('\n');
        let row = 0;
        while (index > 0) {
            if (lines[row].length > index) {
                return [lines[row], index + 1, row + 1];
            }
            index -= lines[row].length + 1;
            row += 1;
        }
        return [lines[row], index + 1, row + 1];
    }
    getPrettyErrorMessage() {
        return `${this.message} (line ${this.row}, col ${this.column})\n${this.line}\n${(this.column ? ' '.repeat(this.column - 1) : '') + '^'}`;
    }
}
exports.ParseError = ParseError;
